/*
# Copyright (C) 2013 audin.
# 2013/07: Modified for LPC1114fn28 demo.
# http://avr.paslog.jp
# This file is licensed under the MIT License.
*/
/*
 * @brief UART interrupt example with ring buffers
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "chip.h"
#include "board.h"
#include "string.h"
#include "xprintf.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

static uint32_t ibaudrate;

/* Transmit and receive ring buffers */
STATIC RINGBUFF_T txring, rxring;

/* Ring buffer size */
#define UART_RB_TX_SIZE 16
#define UART_RB_RX_SIZE 64

/* Transmit and receive buffers */
static uint8_t rxbuff[UART_RB_RX_SIZE], txbuff[UART_RB_TX_SIZE];

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/**
 * @brief	UART interrupt handler using ring buffers
 * @return	Nothing
 */
void UART_IRQHandler(void)
{
	/* Want to handle any errors? Do it here. */

	/* Use default ring buffer handler. Override this with your own
	   code if you need more capability. */
	Chip_UART_IRQRBHandler(LPC_USART, &rxring, &txring);
}

/* 
 * ===  FUNCTION  ======================================================================
 *         Name:  UART_putc
 *  Description:  
 * =====================================================================================
 */
void UART_putc( uint8_t c )
{
	/*  Polling transfer */
	Chip_UART_SendBlocking(LPC_USART, &c,  1);

	/*  Ring buffer transfer . None blocking  
	 *  It is difficult to use this demonstration at this moment. :D 
	 */
	//Chip_UART_SendRB(LPC_USART, &txring, (const uint8_t *) &c, 1);
}

/* 
 * ===  FUNCTION  ======================================================================
 *         Name:  uart_init
 *  Description:  
 * =====================================================================================
 */
void uart_init( uint32_t baudrate)
{

	Board_UART_Init();
	ibaudrate = baudrate;

	/* Setup UART for 8N1 */
	Chip_UART_Init(LPC_USART);
	Chip_UART_SetBaud(LPC_USART, baudrate );
	Chip_UART_ConfigData(LPC_USART, (UART_LCR_WLEN8 | UART_LCR_SBS_1BIT));
	Chip_UART_SetupFIFOS(LPC_USART, (UART_FCR_FIFO_EN | UART_FCR_TRG_LEV2));
	Chip_UART_TXEnable(LPC_USART);

	/* Before using the ring buffers, initialize them using the ring
	   buffer init function */
	RingBuffer_Init(&rxring, rxbuff, 1, UART_RB_RX_SIZE);
	RingBuffer_Init(&txring, txbuff, 1, UART_RB_TX_SIZE);

	/* Enable receive data and line status interrupt */
	Chip_UART_IntEnable(LPC_USART, (UART_IER_RBRINT | UART_IER_RLSINT));

	/* preemption = 1, sub-priority = 1 */
	NVIC_SetPriority(UART0_IRQn, 1);
	NVIC_EnableIRQ(UART0_IRQn);

#ifdef __XPRINTF__
	xfunc_out = UART_putc;  /* connect PutChar function to xprintf() */
#endif

}

/* 
 * ===  FUNCTION  ======================================================================
 *         Name:  uartDrv_getchar
 *  Description:  
 * =====================================================================================
 */
uint8_t	uartDrv_getchar( void ){
	uint8_t c;
	if ( Chip_UART_ReadRB(LPC_USART, &rxring, &c, 1) > 0) {
		return c;
	}
	return 0;		
}

/*******************

 * uartDrv_getBaudrate
  
 *******************/
uint32_t uartDrv_getBaudrate( void )
{
	return ibaudrate;
}


/* 
 * ===  FUNCTION  ======================================================================
 *         Name:  uartIsRxDataAvailable
 *  Description:  
 * =====================================================================================
 */
uint8_t uartIsRxDataAvailable(void){
 	
	return   RingBuffer_GetCount( &rxring ) > 0 ;
}

/**
 * @}
 */
